package edu.unca.cs.csci202;

import java.util.Comparator;
import java.util.LinkedList ;
import java.util.Collections ;
import java.util.Random ;
import processing.core.PApplet;
import static processing.core.PApplet.map;

/**
 *
 * @author J Dean Brock <brock@unca.edu>
 */
public class PDriver extends PApplet {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        PApplet.main("edu.unca.cs.csci202.PDriver");
    }
    
    private static final Comparator[] comps = { 
            new ComparatorVertical(),
            new ComparatorHorizontal(),
            new ComparatorHorizontalThenVertical(),
            new ComparatorHorizontalReversed(),
            new ComparatorDistance(),
            new ComparatorAngle(),
            new ComparatorExciting() } ;    
    
    // Deque does not support clone
    LinkedList<Point1080> rawPoints ;
    
    private static void addRandomPoints(LinkedList<Point1080> l, int n) {
        Random randGen = new Random() ;
        for (int i=0; i<n; ++i) {
            l.push(new Point1080((int)(randGen.nextInt(1920)),
                                 (int)(randGen.nextInt(1080)))) ;
        }
        
    }
    
    private static final int initSize = 16384 ;
    
    
    @Override
    public void setup() {
        rawPoints = new LinkedList<Point1080>() ;
        addRandomPoints(rawPoints, initSize) ;
        ellipseMode(CENTER) ;
        size(1920/2, 1080/2);
        frameRate(10) ;     // default is 10 (I think)
    }

    // Make color i/n through the rainbow
    // Inspired (and confused) by the following blog
    // http://www.markandey.com/2012/04/how-to-generate-rainbow-colors.html
    private static int makeRainbow(int i, int n) {
        double angle = 2*Math.PI*i/n ;
        double rV = Math.cos(angle) ;
        double gV = Math.cos(angle-2*Math.PI/3) ;
        double bV = Math.cos(angle+2*Math.PI/3);
        int rC = (int)map((float)rV, -1.0f, 1.0f, 0, 255) ;
        int gC = (int)map((float)gV, -1.0f, 1.0f, 0, 255) ;
        int bC = (int)map((float)bV, -1.0f, 1.0f, 0, 255) ;
        return 0xFF000000                     // Get ready for CSCI 255
                | ((rC & 0xFF) << 16)
                | ((gC & 0xFF) << 8)
                |  (bC & 0xFF) ;        
    }
    
    private static final int flashRate = 2 ;
    
    @Override
    @SuppressWarnings("unchecked")
    public void draw() {
        LinkedList<Point1080> drawPoints ;
        int pTime = second() / flashRate % (comps.length + 1) ;
        if (pTime == 0) {
            drawPoints = rawPoints ;
        } else {
            drawPoints = (LinkedList<Point1080>) rawPoints.clone() ;
            Collections.sort(drawPoints, (Comparator<Point1080>)comps[pTime-1]) ;
        }
        background(0) ;
        int iMax = drawPoints.size() ;
        int i = 0 ;
        for (Point1080 E : drawPoints) {
            int rV = (int)map(i, 0, iMax, 100, 255) ;
            int gV = (int)map(i, 0, iMax, 100, 255) ;
            int bV = 0 ;
            // fill(rV, gV, bV) ;
            fill(makeRainbow(i, iMax)) ;
            ellipse(E.getX()/2, E.getY()/2, 10, 10) ;
            ++i ;
        }
    }
    
    @Override
    public void mouseClicked() {
        int newX = (int) map(mouseX, 0, width, 0, 1919) ;
        int newY = (int) map(mouseY, 0, height, 0, 1079) ;
        rawPoints.push(new Point1080(newX, newY));
    }

}
